/* reseni soustavy lin. alg. rovnic Gaussovou eliminacni metodou */
/* zadani rovnice v souboru                                      */
/* zamena radku, vyber radku s maximem na hlavni diagonale       */
/* osetreni na singularitu soustavy                              */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

typedef double *pd;

/* Alokace pameti s osetrenim na nedostatek pameti */
void *myall(unsigned kolik)
{
 void *p;
 if ((p=malloc(kolik))==NULL)
  {
   printf("\nNedostatek pameti!\n");
   exit(1);
  }
 return p;
}


/* Vypis soustavy rovnic */
void vyprov(int n, pd a, pd b)
{
 int i,j;
 pd px;
 for (i=0;i<n;i++)
   {
    px=a+n*i;
    for (j=0;j<n;j++)  printf("%+7.3f*%c  ", px[j],('Z'-n+1+j));
    printf("=%+7.3f\n", b[i]);
   }
}


int gauss(int n, pd a, pd b)
{
 int i,j,k;
 pd *mat,pr,px;
 double pom;

 /* Vytvoreni pomocneho pole ukazatelu */
 mat=(pd *)myall(n*sizeof(pd));
 for (i=0;i<n;i++)
   mat[i]=a+n*i;

 /* Primy beh */
 for (k=1;k<n;k++)
   {
    /* hledani maxima a zamena radku */
    for (i=k;i<n;i++)
      if (fabs(mat[i][k-1])>fabs(mat[k-1][k-1]))
	{
	 pr=mat[k-1];
	 mat[k-1]=mat[i];
	 mat[i]=pr;
	 pom=b[k-1];
	 b[k-1]=b[i];
	 b[i]=pom;
	}

    pr=mat[k-1];
    if (pr[k-1]==0) return 0;
    for (i=k;i<n;i++)
      {
       px=mat[i];
       pom=px[k-1]/pr[k-1];
       for (j=k-1;j<n;j++)
	 px[j]-=pom*pr[j];
       b[i]-=pom*b[k-1];
      }
   }

 /* Zpetny beh */
 for (k=n-2;k>=0;k--)
   {
    pr=mat[k+1];
    if (pr[k+1]==0.0) return 0;
    for (i=k;i>=0;i--)
     {
      pom=mat[i][k+1]/pr[k+1];
      for (j=k;j<n;j++)
	mat[i][j]-=pom*pr[j];
      b[i]-=pom*b[k+1];
     }
   }

 /* Konecna uprava */
 for (i=0;i<n;i++)
  {
   b[i]/=mat[i][i];
  }

 free(mat);
 return 1;
}

void main()
{
 int rad;
 int i,j;
 pd A, B;
 FILE *fr;
 double X;
 char s[120];

 /* nacteni soustavy rovnic ze souboru */
 printf("\nZadej jmeno souboru s popisem soustavy rovnic: ");
 gets(s);
 fr=fopen(s, "r");
 if (fr==NULL) return;
 fscanf(fr, "%d", &rad);
 if (rad<2) {fclose(fr); return;}


 A=(pd)myall(rad*rad*sizeof(double));
 B=(pd)myall(rad*sizeof(double));

 for (i=0;i<rad;i++)
   {
    for (j=0;j<rad;j++)
      {
      fscanf(fr, "%lf", &X);
      *(A+rad*i+j)=X;
      }
    fscanf(fr, "%lf", B+i);
   }
 fclose(fr);

 printf("\nZadana soustava rovnic\n");
 vyprov(rad, A, B);

 if (gauss(rad, A, B))
  {
   /* vypise reseni soustavy rovnic */
   printf("Reseni\n");
   for (i=0;i<rad;i++)
     printf("%c=%+7.3f  ", 'Z'-rad+1+i, B[i]);
   printf("\n");
   }
 else
   printf("\nSoustava nema reseni\n");

 for (;getchar()!='\n';);
}
